window.Gleam.initSearch([{"doc":"validated","title":"validated","content":"# validated\n\nEasily accumulate errors in Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/validated)](https://hex.pm/packages/validated)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/validated/)\n\n```gleam\nimport gleam/dict.{type Dict}\nimport validated.{type Validated, Invalid, Valid} as v\n\npub opaque type Config {\n  Config(html: String, manifest: Dict(String, String), hash: String)\n}\n\npub fn new(\n  html_filepath: String,\n  manifest_filepath: String,\n) -&gt; Validated(Config, String) {\n  use html &lt;- v.do(read_file(html_filepath) |&gt; v.string)\n  use data &lt;- v.lazy_guard(read_file(manifest_filepath) |&gt; v.string, empty)\n  use manifest &lt;- v.do(parse_manifest(data) |&gt; v.dict)\n  Valid(Config(html, manifest, hash(data)))\n}\n\npub fn main() {\n  let assert Valid(_) = new(&quot;exists.html&quot;, &quot;exists.json&quot;)\n\n  let assert Invalid(_, [&quot;file not found&quot;, &quot;file_not found&quot;]) =\n    new(&quot;not_there.html&quot;, &quot;not_there.json&quot;)\n\n  let assert Invalid(_, [&quot;file not found&quot;, &quot;failed to parse manifest&quot;]) =\n    new(&quot;not_there.html&quot;, &quot;malformed.json&quot;)\n}\n\nfn empty() -&gt; Config {\n  Config(&quot;&quot;, dict.new(), &quot;&quot;)\n}\nfn read_file(filepath: String) -&gt; Result(String, String) { todo }\nfn parse_manifest(data: String) -&gt; Result(Dict(String, String), String) { todo }\nfn hash(data: String) -&gt; String { todo }\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/validated&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"validated","title":"Validated","content":"pub type Validated(a, e) {\n  Valid(a)\n  Invalid(a, List(e))\n}\n `Validated` represents the result of something that may succeed or not.\n It is similar to `Result`, except that it will accumulate multiple errors\n instead of stopping at the first error.\n `Valid` means it was successful, `Invalid` means it was not successful.\nValid(a)\n\n\nInvalid(a, List(e))\n Invalid requires a default value to enable validation in `use` expressions\n\nSynonyms:\nvalidated.Validated\nvalidated Validated","url":"validated.html#Validated"},{"doc":"validated","title":"Validator","content":"pub type Validator(in, out, error) =\n  fn(in) -&gt; Validated(out, error)\n A function that takes some input and returns a `Validated`\n\nSynonyms:\nvalidated.Validator\nvalidated Validator","url":"validated.html#Validator"},{"doc":"validated","title":"all","content":"pub fn all(\n  validateds: List(Validated(a, b)),\n) -&gt; Validated(List(a), b)\n Combines a list of `Validateds` into a single `Validated`.\n If all elements in the list are `Valid` then the function returns a `Valid`\n holding the list of values.\n Otherwise an `Invalid` is returned that combines all the errors.\n `Valid([])` is returned if the list is empty.\nSynonyms:\nvalidated.all\nvalidated all","url":"validated.html#all"},{"doc":"validated","title":"bit_array","content":"pub fn bit_array(\n  value: Result(BitArray, a),\n) -&gt; Validated(BitArray, a)\n Creates a `Validated` from a `Result`.\n It uses an empty `BitArray` as the default value in case of failure.\nSynonyms:\nvalidated.bit_array\nvalidated bit_array","url":"validated.html#bit_array"},{"doc":"validated","title":"bool","content":"pub fn bool(value: Result(Bool, a)) -&gt; Validated(Bool, a)\n Creates a `Validated` from a `Result`.\n It uses `False` as the default value in case of failure.\nSynonyms:\nvalidated.bool\nvalidated bool","url":"validated.html#bool"},{"doc":"validated","title":"dict","content":"pub fn dict(\n  value: Result(Dict(a, b), c),\n) -&gt; Validated(Dict(a, b), c)\n Creates a `Validated` from a `Result`.\n It uses an empty `Dict` as the default value in case of failure.\nSynonyms:\nvalidated.dict\nvalidated dict","url":"validated.html#dict"},{"doc":"validated","title":"do","content":"pub fn do(\n  validated: Validated(a, b),\n  next: fn(a) -&gt; Validated(c, b),\n) -&gt; Validated(c, b)\n &quot;Updates&quot; a `Valid` value by passing its value to a function that yields a `Validated`,\n and returning the yielded `Validated`. (This may &quot;replace&quot; the `Valid` with an `Invalid`.)\n\n If the input is an `Invalid` rather than an `Valid`, the function is still called using the default\n value from the `Invalid`. If the function succeeds, the `Invalid`&#39;s default is replaced with the `Valid` value.\n If the function fails, the first `Invalid` errors are combined with the returned `Invalid` errors.\n\n This function is useful in `use` expressions to ergonomically validate fields of a record\n\n ## Examples\n\n \n use a &lt;- do(Valid(1))\n use b &lt;- do(Valid(2))\n Valid(#(a, b))\n // -&gt; Valid(#(1, 2))\n \n\n \n use a &lt;- do(Invalid(0, [Nil]))\n use b &lt;- do(Valid(2))\n use c &lt;- do(Invalid(0, [Nil]))\n Valid(#(a, b, c))\n // -&gt; Invalid(#(0, 2, 0), [Nil, Nil])\n \nSynonyms:\nvalidated.do\nvalidated do","url":"validated.html#do"},{"doc":"validated","title":"flatten","content":"pub fn flatten(\n  validated: Validated(Validated(a, b), b),\n) -&gt; Validated(a, b)\n Merges a nested `Validated` into a single layer.\nSynonyms:\nvalidated.flatten\nvalidated flatten","url":"validated.html#flatten"},{"doc":"validated","title":"float","content":"pub fn float(value: Result(Float, a)) -&gt; Validated(Float, a)\n Creates a `Validated` from a `Result`.\n It uses `0.0` as the default value in case of failure.\nSynonyms:\nvalidated.float\nvalidated float","url":"validated.html#float"},{"doc":"validated","title":"guard","content":"pub fn guard(\n  validated: Validated(a, b),\n  default: c,\n  continue: fn(a) -&gt; Validated(c, b),\n) -&gt; Validated(c, b)\n Like `do`, but does not continue accumulating further errors if the\n `Validated` is `Invalid`.\nSynonyms:\nvalidated.guard\nvalidated guard","url":"validated.html#guard"},{"doc":"validated","title":"int","content":"pub fn int(value: Result(Int, a)) -&gt; Validated(Int, a)\n Creates a `Validated` from a `Result`.\n It uses `0` as the default value in case of failure.\nSynonyms:\nvalidated.int\nvalidated int","url":"validated.html#int"},{"doc":"validated","title":"is_invalid","content":"pub fn is_invalid(validated: Validated(a, b)) -&gt; Bool\n Checks whether the `Validated` is `Invalid`\nSynonyms:\nvalidated.is_invalid\nvalidated is_invalid","url":"validated.html#is_invalid"},{"doc":"validated","title":"is_valid","content":"pub fn is_valid(validated: Validated(a, b)) -&gt; Bool\n Checks whether the `Validated` is `Valid`\nSynonyms:\nvalidated.is_valid\nvalidated is_valid","url":"validated.html#is_valid"},{"doc":"validated","title":"lazy_guard","content":"pub fn lazy_guard(\n  validated: Validated(a, b),\n  default: fn() -&gt; c,\n  continue: fn(a) -&gt; Validated(c, b),\n) -&gt; Validated(c, b)\n Like `guard` but accepts a callback for the default value in case the\n `Validated` is `Invalid`.\nSynonyms:\nvalidated.lazy_guard\nvalidated lazy_guard","url":"validated.html#lazy_guard"},{"doc":"validated","title":"list","content":"pub fn list(value: Result(List(a), b)) -&gt; Validated(List(a), b)\n Creates a `Validated` from a `Result`.\n It uses an empty list as the default value in case of failure.\nSynonyms:\nvalidated.list\nvalidated list","url":"validated.html#list"},{"doc":"validated","title":"map","content":"pub fn map(\n  validated: Validated(a, b),\n  f: fn(a) -&gt; c,\n) -&gt; Validated(c, b)\n Updates a value held within the `Valid` of a `Validated` by calling a given function on it.\n If the `Validated` is an `Invalid` rather than `Valid`, the function is called on the default value.\nSynonyms:\nvalidated.map\nvalidated map","url":"validated.html#map"},{"doc":"validated","title":"map_error","content":"pub fn map_error(\n  validated: Validated(a, b),\n  f: fn(List(b)) -&gt; List(c),\n) -&gt; Validated(a, c)\n Updates a value held within the `Invalid` of a `Validated` by calling a given function on it.\n If the result is `Valid` rather than `Invalid` the function is not called and the result stays the same.\nSynonyms:\nvalidated.map_error\nvalidated map_error","url":"validated.html#map_error"},{"doc":"validated","title":"optional","content":"pub fn optional(\n  value: Result(Option(a), b),\n) -&gt; Validated(Option(a), b)\n Creates a `Validated` from a `Result`.\n It uses `None` as the default value in case of failure.\nSynonyms:\nvalidated.optional\nvalidated optional","url":"validated.html#optional"},{"doc":"validated","title":"replace","content":"pub fn replace(\n  validated: Validated(a, b),\n  value: c,\n) -&gt; Validated(c, b)\n Replace the value within a `Validated`\nSynonyms:\nvalidated.replace\nvalidated replace","url":"validated.html#replace"},{"doc":"validated","title":"result","content":"pub fn result(value: Result(a, b), default: a) -&gt; Validated(a, b)\n Creates a `Validated` from a `Result`.\n It requires a default value in case of failure.\nSynonyms:\nvalidated.result\nvalidated result","url":"validated.html#result"},{"doc":"validated","title":"run","content":"pub fn run(\n  validator: fn(a) -&gt; Validated(b, c),\n  input: a,\n) -&gt; Validated(b, c)\n Run a `Validator` function.\nSynonyms:\nvalidated.run\nvalidated run","url":"validated.html#run"},{"doc":"validated","title":"run_all","content":"pub fn run_all(\n  validators: List(fn(a) -&gt; Validated(b, c)),\n  input: a,\n) -&gt; Validated(Nil, c)\n Run all the `Validators` in order on the given input.\n It will accumulate all the errors from all of the `Validators`.\n\n If there are no errors, or if the list is empty, `Valid(Nil)` is returned.\nSynonyms:\nvalidated.run_all\nvalidated run_all","url":"validated.html#run_all"},{"doc":"validated","title":"string","content":"pub fn string(value: Result(String, a)) -&gt; Validated(String, a)\n Creates a `Validated` from a `Result`.\n It uses an empty string as the default value in case of failure.\nSynonyms:\nvalidated.string\nvalidated string","url":"validated.html#string"},{"doc":"validated","title":"to_result","content":"pub fn to_result(\n  validated: Validated(a, b),\n) -&gt; Result(a, List(b))\n Convert a `Validated` into a `Result`\nSynonyms:\nvalidated.to_result\nvalidated to_result","url":"validated.html#to_result"},{"doc":"validated","title":"unwrap","content":"pub fn unwrap(validated: Validated(a, b)) -&gt; a\n Return the `Valid` value of the validated, or the default value if it is `Invalid`.\nSynonyms:\nvalidated.unwrap\nvalidated unwrap","url":"validated.html#unwrap"},{"doc":"validated","title":"validated","content":" This module provides the `Validated` type and associated functions\n to accumulate errors in an ergonomic way.\n\n ## Example\n ```gleam\n fn validate_form(email: String, age: Int) -&gt; Validated(Form, String) {\n   use email &lt;- v.do(validate_email(email))\n   use age &lt;- v.do(validate_age(age))\n   Valid(Form(email:, age:))\n }\n\n validated_form(&quot;lucy@example.com&quot;, 20)\n // -&gt; Valid(Form(&quot;lucy@example.com&quot;, 20))\n\n validated_form(&quot;asdf&quot;, 5)\n // -&gt; Invalid(Form(&quot;&quot;, 0), [&quot;not a valid email&quot;, &quot;must be 18 or older&quot;])\n ```\n\n This API is possible because a `Validated` requires a &quot;default&quot; value\n in case of failure. That way, the default value is passed through to\n continue the validation.\n\n Since a value within a validation block may or may not be the default\n value, it is important to **never perform side-effects inside the\n validation**.","url":"validated.html"}]);